<?php

if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');

function autoupdate_check_version($version=PHPWG_VERSION)
{
  global $conf;

  if (!isset($_SESSION['need_update']))
  {
    $_SESSION['need_update'] = null;

    if (preg_match('/(\d+\.\d+)\.(\d+)/', $version, $matches)
      and @fetchRemote(PHPWG_URL.'/download/all_versions.php', $result))
    {
      $all_versions = @explode("\n", $result);
      $new_version = trim($all_versions[0]);
      $_SESSION['need_update'] = version_compare($version, $new_version, '<');
    }
  }

  if ($_SESSION['need_update'])
  {
    return l10n('A new version of Piwigo is available.').'<br>'
          . '<a href="admin.php?page=plugin-autoupdate">'.l10n('Click here to upgrade automatically').'</a>';
  }
  else
  {
    // Gallery is up to date
    // Check plugins upgrade
    include_once(PHPWG_ROOT_PATH.'admin/include/plugins.class.php');
    include_once(PHPWG_ROOT_PATH.'admin/include/themes.class.php');
    include_once(PHPWG_ROOT_PATH.'admin/include/languages.class.php');
    $plugins = new plugins();
    $themes = new themes();
    $languages = new languages();
    $ignore_list = unserialize($conf['autoupdate_ignore_list']);

    // Upgrade?
    if (!isset($ignore_list['plugins']) or !isset($ignore_list['themes']) or !isset($ignore_list['languages']))
    {
      $ignore_list = array('plugins'=>array(),'themes'=>array(),'languages'=>array());
      $conf['autoupdate_ignore_list'] = serialize($ignore_list);
      conf_update_param('autoupdate_ignore_list', pwg_db_real_escape_string($conf['autoupdate_ignore_list']));
    }

    if (!isset($_SESSION['extensions_need_update']))
    {
      $_SESSION['extensions_need_update'] = null;
      autoupdate_check_plugins_upgrade($plugins, $ignore_list);
      autoupdate_check_themes_upgrade($themes, $ignore_list);
      autoupdate_check_languages_upgrade($languages, $ignore_list);
    }
    else
    {
      if (!empty($_SESSION['extensions_need_update']['plugins']))
      {
        // Check if plugins have been upgraded since last check
        foreach($plugins->fs_plugins as $plugin_id => $fs_plugin)
        {
          if (isset($_SESSION['extensions_need_update']['plugins'][$plugin_id])
            and $plugins->plugin_version_compare($fs_plugin['version'], $_SESSION['extensions_need_update']['plugins'][$plugin_id]))
          {
            // Plugin have been updated
            autoupdate_check_extensions_upgrade($plugins, $themes, $languages, $ignore_list);
          }
        }
      }
      if (!empty($_SESSION['extensions_need_update']['themes']))
      {
        // Check if themes have been upgraded since last check
        foreach($themes->fs_themes as $theme_id => $fs_theme)
        {
          if (isset($_SESSION['extensions_need_update']['themes'][$theme_id])
            and $themes->theme_version_compare($fs_theme['version'], $_SESSION['extensions_need_update']['themes'][$theme_id]))
          {
            // theme have been updated
            autoupdate_check_extensions_upgrade($themes, $themes, $languages, $ignore_list);
          }
        }
      }
      if (!empty($_SESSION['extensions_need_update']['languages']))
      {
        // Check if languages have been upgraded since last check
        foreach($languages->fs_languages as $language_id => $fs_language)
        {
          if (isset($_SESSION['extensions_need_update']['languages'][$language_id])
            and $languages->language_version_compare($fs_language['version'], $_SESSION['extensions_need_update']['languages'][$language_id]))
          {
            // language have been updated
            autoupdate_check_extensions_upgrade($languages, $themes, $languages, $ignore_list);
          }
        }
      }
    }

    if (!empty($_SESSION['extensions_need_update']))
    {
      return l10n('Some upgrades are available for you plugins').'<br>'
        . '<a href="admin.php?page=plugin-autoupdate">'.l10n('Click here see upgrade plugins page').'</a>';
    }
  }

  if ($_SESSION['need_update'] === false and $_SESSION['extensions_need_update'] === array())
  {
    return l10n('Gallery and plugins are up to date');
  }
  else
  {
    return l10n('Unable to check upgrades...')
      . '<br><a href="admin.php?action=check_autoupdate">'
      . l10n('Click here to check upgrades now') . '</a>';
  }
}

function autoupdate_check_plugins_upgrade($plugins, $ignore_list)
{
  global $conf;

  if ($plugins->get_server_plugins())
  {
    $plugins_ignore_list = array();
    $need_upgrade = array();
    if (is_null($_SESSION['extensions_need_update']))
      $_SESSION['extensions_need_update'] = array();

    foreach($plugins->fs_plugins as $plugin_id => $fs_plugin)
    {
      if (isset($fs_plugin['extension']) and isset($plugins->server_plugins[$fs_plugin['extension']]))
      {
        $plugin_info = $plugins->server_plugins[$fs_plugin['extension']];

        if (!$plugins->plugin_version_compare($fs_plugin['version'], $plugin_info['revision_name']))
        {
          $fs_plugin_name = htmlspecialchars_decode($fs_plugin['name']);

          if (in_array($fs_plugin_name, $ignore_list['plugins']))
          {
            array_push($plugins_ignore_list, $fs_plugin_name);
          }
          else
          {
            $need_upgrade[$plugin_id] = $plugin_info['revision_name'];
          }
        }
      }
    }
    if (!empty($need_upgrade))
    {
      $_SESSION['extensions_need_update']['plugins'] = $need_upgrade;
    }
    // Update ignore list in database
    $conf['autoupdate_ignore_list']['plugins'] = $plugins_ignore_list;
    conf_update_param('autoupdate_ignore_list', pwg_db_real_escape_string(serialize($conf['autoupdate_ignore_list'])));
  }
}

function autoupdate_check_themes_upgrade($themes, $ignore_list)
{
  global $conf;

  if ($themes->get_server_themes())
  {
    $themes_ignore_list = array();
    $need_upgrade = array();
    if (is_null($_SESSION['extensions_need_update']))
      $_SESSION['extensions_need_update'] = array();

    foreach($themes->fs_themes as $theme_id => $fs_theme)
    {
      if (isset($fs_theme['extension']) and isset($themes->server_themes[$fs_theme['extension']]))
      {
        $theme_info = $themes->server_themes[$fs_theme['extension']];

        if (!$themes->theme_version_compare($fs_theme['version'], $theme_info['revision_name']))
        {
          $fs_theme_name = htmlspecialchars_decode($fs_theme['name']);

          if (in_array($fs_theme_name, $ignore_list['theme']))
          {
            array_push($themes_ignore_list, $fs_theme_name);
          }
          else
          {
            $need_upgrade[$theme_id] = $theme_info['revision_name'];
          }
        }
      }
    }
    if (!empty($need_upgrade))
    {
      $_SESSION['extensions_need_update']['themes'] = $need_upgrade;
    }
    // Update ignore list in database
    $conf['autoupdate_ignore_list']['themes'] = $themes_ignore_list;
    conf_update_param('autoupdate_ignore_list', pwg_db_real_escape_string(serialize($conf['autoupdate_ignore_list'])));
  }
}

function autoupdate_check_languages_upgrade($plugins, $ignore_list)
{
  global $conf;

  if ($languages->get_server_languages())
  {
    $languages_ignore_list = array();
    $need_upgrade = array();
    if (is_null($_SESSION['extensions_need_update']))
      $_SESSION['extensions_need_update'] = array();

    foreach($languages->fs_languages as $language_id => $fs_language)
    {
      if (isset($fs_language['extension']) and isset($languages->server_languages[$fs_language['extension']]))
      {
        $language_info = $languages->server_languages[$fs_language['extension']];

        if (!$languages->language_version_compare($fs_language['version'], $language_info['revision_name']))
        {
          $fs_language_name = htmlspecialchars_decode($fs_language['name']);

          if (in_array($fs_language_name, $ignore_list['languages']))
          {
            array_push($languages_ignore_list, $fs_language_name);
          }
          else
          {
            $need_upgrade[$language_id] = $language_info['revision_name'];
          }
        }
      }
    }
    if (!empty($need_upgrade))
    {
      $_SESSION['extensions_need_update']['languages'] = $need_upgrade;
    }
    // Update ignore list in database
    $conf['autoupdate_ignore_list']['languages'] = $languages_ignore_list;
    conf_update_param('autoupdate_ignore_list', pwg_db_real_escape_string(serialize($conf['autoupdate_ignore_list'])));
  }
}

?>